/*
Google Music Frame

Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

This program is free software: you can redistribute it and/or modify it 
under the terms of the GNU General Public License version 3, as published 
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/




/* used to propagate exception to main program */
var $throw_exception = function(message){
	console.log("GMF::exception::" + message);
}

try{
	/*
	 * WebKit.WebView.console_message signal doesn't provide 
	 * debug/error/warn flag for console.debug/error/warn().
	 * Therefore we use these wrappers.
	 */
	var $log_debug = function(message){
		console.log("GMF::debug::" + message);
	}
	
	var $log_info = function(message){
		console.log("GMF::info::" + message);
	}
	
	var $log_warn = function(message){
		console.log("GMF::warn::" + message);
	}
	
	var $log_error = function(message){
		console.log("GMF::error::" + message);
	}
	
	
	var $GMF = function(){
		this.SEPARATOR = "§#@!@#§";
		this.changed_callbacks = [];
		this.album_art = null;
		this.artist = null;
		this.song = null;
		this.state = "none";
		this.can_prev = false;
		this.can_next = false;
		this.timeout = null;
		
		this.iframe = document.createElement("iframe");
		this.iframe.name = "GMF_iframe";
		this.iframe.style.display = "none";
		document.body.appendChild(this.iframe);
		this.changed_callbacks.push(this.saveState.bind(this));
		this.update();
	};
	
	$GMF.prototype.update = function(){
		if(this.timeout) clearTimeout(this.timeout);
		try{
			var album_art = document.getElementById('playingAlbumArt').src;
			var artist = document.getElementById('playerArtist').firstChild.title;
			var song = document.getElementById('playerSongTitle').firstChild.title;
		}
		catch(e){
			var album_art = null;
			var artist = null;
			var song = null;
		}
		
		try{
			var play_pause =  document.getElementById('playPause');
			var pp_pressed = play_pause.getAttribute("aria-pressed");
			var pp_disabled = play_pause.getAttribute("aria-disabled");
			var state;
			if(pp_pressed === null && pp_disabled == null){
				state = "none";
			}
			else if(pp_disabled == "true"){
				state = "none";
			}
			else if(pp_pressed === null && pp_disabled == "false"){
				state = "paused";
			}
			else{
				state = ( pp_pressed == "true") ? "playing" : "paused";
			}
			
			if(state != "none"){
				var disabled = document.getElementById('rew').getAttribute("aria-disabled")
				this.can_prev = disabled !== null && disabled !== "true";
				disabled = document.getElementById('ff').getAttribute("aria-disabled")
				this.can_next = disabled !== null && disabled !== "true";
			}
			else{
				this.can_prev = this.can_next = false;
			}
			
		}
		catch(e){
			var state = "none";
			console.log(e.message);
		}
		
		
		var changed = false;
		if(state !== this.state){
			this.state = state;
			changed = true;
		}
		if(album_art !== this.album_art){
			this.album_art = album_art;
			changed = true;
		}
		if(artist !== this.artist){
			this.artist = artist;
			changed = true;
		}	
		if(song !== this.song){
			this.song = song;
			changed = true;
		}
		
		if(changed){
			for(var key in this.changed_callbacks){
				this.changed_callbacks[key]();
			}
		}
		//~ console.log(this.state + ", " + this.can_next + ", " + this.can_prev + ", " + this.album_art + ", " + this.artist + ", " + this.song);
		this.timeout = setTimeout(this.update.bind(this), 500);
		
	}
	
	$GMF.prototype.saveState = function(){
		var state = this.state
			+ this.SEPARATOR + (this.can_prev ? "true" : "false")
			+ this.SEPARATOR + (this.can_next ? "true" : "false");
		if(this.album_art !== null || this.artist !== null || this.song !== null){
			state += this.SEPARATOR + (this.album_art === null ? "" : this.album_art)
				+ this.SEPARATOR + (this.artist === null ? "" : this.artist)
				+ this.SEPARATOR + (this.song === null ? "" : this.song);
		}
		$log_debug("JS State: " + state);
		var doc = this.iframe.contentDocument;
		if(doc.title != state) doc.title = state;
	}
	
	$GMF.prototype.command = function(cmd){
		try{
			SJBpost(cmd);
			//~ this.update();
		}
		catch(e){
			$throw_exception(e.message);
		}
	}
	
	$GMF.prototype._triggerMouseEvent = function(elm, name){
	    var event = document.createEvent('MouseEvents');
	    event.initMouseEvent(name, true, true, document.defaultView, 1, 0, 0, 0, 0, false, false, false, false, 0, elm);
	    elm.dispatchEvent(event);
	}

	$GMF.prototype._clickOnElement = function(elm){
	    this._triggerMouseEvent(elm, 'mouseover');
	    this._triggerMouseEvent(elm, 'mousedown');
	    this._triggerMouseEvent(elm, 'mouseup');
	}

	$GMF.prototype.thumbsUp = function(){
		this._clickOnElement(document.getElementById('thumbsUpPlayer'));
	}

	$GMF.prototype.thumbsDown = function(){
	    this._clickOnElement(document.getElementById('thumbsDownPlayer'));
	}

	
	$GMF = new $GMF(); // Singleton ;-)
}
catch(e){
	$throw_exception(e.message);
}
